@@ -2,6 +2,7 @@ use crate::common::InputMode;
 use crate::csv::Row;
 use crate::find;
 use crate::sort;
+use crate::sort::SortOrder;
 use crate::view;
 use crate::view::Header;
 use crate::wrap;
@@ -106,17 +107,66 @@ impl<'a> CsvTable<'a> {
             }
         }
 
+        // Limit maximum width for a column to make way for other columns
+        let max_single_column_width = (area_width as f32 * MAX_COLUMN_WIDTH_FRACTION) as u16;
+        let mut clipped_columns: Vec<(usize, u16)> = vec![];
         for (i, w) in column_widths.iter_mut().enumerate() {
             if overriden_indices.contains(&self.header.get(i).unwrap().origin_index) {
                 *w = max(*w, NUM_SPACES_BETWEEN_COLUMNS);
             } else {
                 *w += NUM_SPACES_BETWEEN_COLUMNS;
-                *w = min(*w, (area_width as f32 * MAX_COLUMN_WIDTH_FRACTION) as u16);
+                if *w > max_single_column_width {
+                    clipped_columns.push((i, *w));
+                    *w = max_single_column_width;
+                }
             }
         }
+
+        // If clipping was too aggressive, redistribute the remaining width
+        CsvTable::redistribute_widths_after_clpping(
+            &mut column_widths,
+            area_width,
+            clipped_columns,
+        );
+
         column_widths
     }
 
+    fn redistribute_widths_after_clpping(
+        column_widths: &mut [u16],
+        area_width: u16,
+        mut clipped_columns: Vec<(usize, u16)>,
+    ) {
+        if clipped_columns.is_empty() {
+            // Nothing to adjust
+            return;
+        }
+
+        let total_width: u16 = column_widths.iter().sum();
+        if total_width >= area_width {
+            // No need to adjust if we're already using the full width
+            return;
+        }
+
+        // Greedily adjust from the narrowest column by equally distributing the remaining width. If
+        // a column doesn't use the allocated adjustment, subsequent columns will get to use it.
+        clipped_columns.sort_by_key(|x| x.1);
+
+        // Subtract 1 to leave space for the right border. If not, this will be too greedy and
+        // consume all the space making that border disappear.
+        let mut remaining_width = area_width.saturating_sub(total_width).saturating_sub(1);
+
+        let mut num_columns_to_adjust = clipped_columns.len();
+        for (i, width_before_clipping) in clipped_columns {
+            let adjustment = remaining_width / num_columns_to_adjust as u16;
+            let width_after_adjustment = min(width_before_clipping, column_widths[i] + adjustment);
+            let added_width = width_after_adjustment - column_widths[i];
+            column_widths[i] = width_after_adjustment;
+            remaining_width -= added_width;
+            num_columns_to_adjust -= 1;
+        }
+    }
+
     fn get_row_heights(
         &self,
         area_height: u16,
@@ -180,11 +230,7 @@ impl<'a> CsvTable<'a> {
         area: Rect,
         rows: &[Row],
         view_layout: &ViewLayout,
-    ) -> u16 {
-        // TODO: better to determine width from total number of records, so this is always fixed
-        let max_row_num = rows.iter().map(|x| x.record_num).max().unwrap_or(0);
-        let mut section_width = format!("{max_row_num}").len() as u16;
-
+    ) {
         // Render line numbers
         let y_first_record = area.y;
         let mut y = area.y;
@@ -199,23 +245,17 @@ impl<'a> CsvTable<'a> {
                 }
             }
             let span = Span::styled(row_num_formatted, style);
-            buf.set_span(0, y, &span, section_width);
+            buf.set_span(0, y, &span, view_layout.row_number_layout.max_length);
             y += view_layout.row_heights[i];
             if y >= area.bottom() {
                 break;
             }
         }
-        section_width = section_width + NUM_SPACES_AFTER_LINE_NUMBER + 1; // one char reserved for line; add one for symmetry
 
         state.borders_state = Some(BordersState {
-            x_row_separator: section_width,
+            x_row_separator: view_layout.row_number_layout.x_row_separator,
             y_first_record,
         });
-
-        // Add more space before starting first column
-        section_width += NUM_SPACES_AFTER_LINE_NUMBER;
-
-        section_width
     }
 
     fn render_header_borders(&self, buf: &mut Buffer, area: Rect) -> (u16, u16) {
@@ -240,6 +280,10 @@ impl<'a> CsvTable<'a> {
         let y_first_record = borders_state.y_first_record;
         let section_width = borders_state.x_row_separator;
 
+        if area.width < section_width {
+            return;
+        }
+
         let line_number_block = Block::default()
             .borders(Borders::RIGHT)
             .border_style(Style::default().fg(Color::Rgb(64, 64, 64)));
@@ -284,7 +328,11 @@ impl<'a> CsvTable<'a> {
     fn get_effective_column_name(&self, column_name: &str, sorter_state: &SorterState) -> String {
         if let SorterState::Enabled(info) = sorter_state {
             if info.status == sort::SorterStatus::Finished && info.column_name == column_name {
-                return format!("{} [▾]", column_name);
+                let indicator = match info.order {
+                    SortOrder::Ascending => "▴",
+                    SortOrder::Descending => "▾",
+                };
+                return format!("{} [{}]", column_name, indicator);
             }
         }
         column_name.to_string()
@@ -660,9 +708,15 @@ impl<'a> CsvTable<'a> {
         buf.set_span(area.x, area.bottom().saturating_sub(1), &span, area.width);
     }
 
-    fn get_view_layout(&self, area: Rect, state: &mut CsvTableState) -> ViewLayout {
+    fn get_view_layout(&self, area: Rect, state: &mut CsvTableState, rows: &[Row]) -> ViewLayout {
+        let max_row_num = rows.iter().map(|x| x.record_num).max().unwrap_or(0);
+        let max_row_num_length = format!("{max_row_num}").len() as u16;
+        let row_num_section_width_with_spaces =
+            max_row_num_length + 2 * NUM_SPACES_AFTER_LINE_NUMBER + 1;
+        let x_row_separator = max_row_num_length + NUM_SPACES_AFTER_LINE_NUMBER + 1;
+
         let column_widths = self.get_column_widths(
-            area.width,
+            area.width.saturating_sub(row_num_section_width_with_spaces),
             &state.column_width_overrides,
             &state.sorter_state,
         );
@@ -676,11 +730,16 @@ impl<'a> CsvTable<'a> {
         );
         state.num_cols_rendered = 0;
         state.col_ending_pos_x = 0;
-        // state.debug = format!("get_row_heights elapsed: {:?}", _tic.elapsed());
-        // state.debug = format!("row_heights: {:?}, area_height: {:?}", row_heights, area.height);
+
+        let row_number_layout = RowNumberLayout {
+            max_length: max_row_num_length,
+            width_with_spaces: row_num_section_width_with_spaces,
+            x_row_separator,
+        };
         ViewLayout {
             column_widths,
             row_heights,
+            row_number_layout,
         }
     }
 }
@@ -697,7 +756,7 @@ impl<'a> StatefulWidget for CsvTable<'a> {
 
         let status_height = 2;
 
-        let layout = self.get_view_layout(area, state);
+        let layout = self.get_view_layout(area, state, self.rows);
         state.view_layout = Some(layout.clone());
 
         let (y_header, y_first_record) = self.render_header_borders(buf, area);
@@ -712,8 +771,8 @@ impl<'a> StatefulWidget for CsvTable<'a> {
                 .saturating_sub(status_height),
         );
 
-        let row_num_section_width =
-            self.render_row_numbers(buf, state, rows_area, self.rows, &layout);
+        self.render_row_numbers(buf, state, rows_area, self.rows, &layout);
+        let row_num_section_width = layout.row_number_layout.width_with_spaces;
 
         self.render_row(
             buf,
@@ -782,10 +841,18 @@ struct FillerStyle {
     short_padding: bool,
 }
 
-#[derive(Clone)]
+#[derive(Debug, Clone)]
+pub struct RowNumberLayout {
+    pub max_length: u16,
+    pub width_with_spaces: u16,
+    pub x_row_separator: u16,
+}
+
+#[derive(Debug, Clone)]
 pub struct ViewLayout {
     pub column_widths: Vec<u16>,
     pub row_heights: Vec<u16>,
+    pub row_number_layout: RowNumberLayout,
 }
 
 impl ViewLayout {
@@ -936,10 +1003,11 @@ enum SorterState {
 }
 
 impl SorterState {
-    fn from_sorter(sorter: &sort::Sorter) -> Self {
+    fn from_sorter(sorter: &sort::Sorter, sort_order: SortOrder) -> Self {
         Self::Enabled(SorterInfo {
             status: sorter.status(),
             column_name: sorter.column_name().to_string(),
+            order: sort_order,
         })
     }
 }
@@ -947,6 +1015,7 @@ impl SorterState {
 struct SorterInfo {
     status: sort::SorterStatus,
     column_name: String,
+    order: SortOrder,
 }
 
 impl SorterInfo {
@@ -1122,9 +1191,9 @@ impl CsvTableState {
             + NUM_SPACES_AFTER_LINE_NUMBER
     }
 
-    pub fn update_sorter(&mut self, sorter: &Option<Arc<sort::Sorter>>) {
+    pub fn update_sorter(&mut self, sorter: &Option<Arc<sort::Sorter>>, sort_order: SortOrder) {
         if let Some(s) = sorter {
-            self.sorter_state = SorterState::from_sorter(s.as_ref());
+            self.sorter_state = SorterState::from_sorter(s.as_ref(), sort_order);
         } else {
             self.sorter_state = SorterState::Disabled;
         }
